#![allow(unused_variables)]
use std::collections::HashMap;

use crate::checked::Type;

#[derive(Clone, PartialEq, Debug)]
pub struct Sym {
    pub id: u64,
    pub typ: Type,
}

pub struct Syms<'syms> {
    parent: Option<&'syms Syms<'syms>>,
    map: HashMap<u64, Sym>,
}

impl<'syms> Syms<'syms> {
    pub fn new() -> Self {
        Self {
            parent: None,
            map: HashMap::new(),
        }
    }

    pub fn child(&'syms self) -> Self {
        Self {
            parent: Some(self),
            map: HashMap::new(),
        }
    }

    pub fn get(&self, id: u64) -> Option<Sym> {
        if let Some(sym) = self.map.get(&id) {
            return Some(sym.clone());
        }
        if let Some(parent) = self.parent {
            return parent.get(id);
        }
        None
    }

    pub fn defined_locally(&self, id: u64) -> bool {
        self.map.contains_key(&id)
    }

    pub fn define(&mut self, id: u64, typ: Type) {
        self.map.insert(id, Sym { id, typ });
    }
}
